package com.shanghaichengdi.ghjgitem.controller;

import com.alibaba.fastjson.JSONObject;
import com.shanghaichengdi.ghjgitem.domain.Video;
import com.shanghaichengdi.ghjgitem.domain.WarningType;
import com.shanghaichengdi.ghjgitem.service.IAccidentPreWarningService;
import com.shanghaichengdi.ghjgitem.util.CommonTools;
import com.shanghaichengdi.ghjgitem.vo.*;
import com.shanghaigeography.entity.ResultMsg;
import com.shanghaigeography.eum.ResultState;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

/**
 * @author gengaoliu
 */
@Api("卡口预警管理")
@RestController
@CrossOrigin
@RequestMapping("/ghjg/AccidentPreWarning")
@Slf4j
public class AccidentPreWarningController {
    @Resource
    private IAccidentPreWarningService service;

    @PostMapping("GetWarningType")
    @ApiOperation("获取【预警类型】字典")
    @ResponseBody
    public String getWarningType() {
        List<WarningType> typeList = service.getWarningType();
        if (CollectionUtils.isEmpty(typeList)) {
            return new ResultMsg<String>().setCode(ResultState.ERROR_NEEDDATA).setMsg("暂无数据").toJSON();
        }
        return new ResultMsg<String>()
                .setCode(ResultState.SUCCESS)
                .setData(JSONObject.toJSONString(typeList))
                .toJSON();
    }

    @PostMapping("getAllPage")
    @ApiOperation("卡口预警管理列表")
    @ResponseBody
    public String getAllPage(@RequestBody Map<String, Object> map) {
        List<AccidentPreWarningVo> voList = service.getAllPage(map);
        int count = CommonTools.getTotal(voList);
        if (CollectionUtils.isEmpty(voList)) {
            return new ResultMsg<String>()
                    .setCode(ResultState.ERROR_NEEDDATA)
                    .setMsg("暂无数据")
                    .setData(null)
                    .toJSON();
        }
        return new ResultMsg<String>()
                .setCode(ResultState.SUCCESS)
                .setMsg(String.valueOf(count))
                .setData(JSONObject.toJSONString(voList))
                .toJSON();
    }

    @GetMapping("getBoatDetails/{boatName}")
    @ApiOperation("船舶信息详情")
    public String getBoatDetails(@PathVariable("boatName") String boatName) {
        Optional<BoatInfoDetailsVo> voOptional = service.getBoatDetails(boatName);
        if (!voOptional.isPresent()) {
            return new ResultMsg<String>().setCode(ResultState.ERROR_NEEDDATA).setMsg("暂无数据").toJSON();
        }
        return new ResultMsg<String>()
                .setCode(ResultState.SUCCESS)
                .setData(JSONObject.toJSONString(voOptional.get()))
                .toJSON();
    }

    @GetMapping("getWarnDetails/{warnCode}")
    @ApiOperation("预警信息详情")
    public String getWarnDetails(@PathVariable("warnCode") String warnCode) {
        Optional<WarnningDetailsVo> mapOptional = service.getWarnDetails(warnCode);
        if (!mapOptional.isPresent()) {
            return new ResultMsg<String>().setCode(ResultState.ERROR_NEEDDATA).setMsg("暂无数据").toJSON();
        }
        return new ResultMsg<String>()
                .setCode(ResultState.SUCCESS)
                .setData(JSONObject.toJSONString(mapOptional.get()))
                .toJSON();
    }

    @GetMapping("getPicTureByCode/{warnCode}")
    @ApiOperation("查看图片")
    public String getPicTureByCode(@PathVariable("warnCode") String warnCode) {
        Optional<Video> videoOptional = service.getPicTureByCode(warnCode);
        if (!videoOptional.isPresent()) {
            return new ResultMsg<String>().setCode(ResultState.ERROR_NEEDDATA).setMsg("该预警暂无图片数据").toJSON();
        }
        String photoName = videoOptional.get().getPhotoname();
        String[] split = photoName.split(";");
        List<String> photoUrl = Arrays.stream(split).collect(Collectors.toList());
        return new ResultMsg<String>()
                .setCode(ResultState.SUCCESS)
                .setData(JSONObject.toJSONString(photoUrl))
                .toJSON();
    }

    @GetMapping("getReportDetails/{boatName}")
    @ApiOperation("船舶报港信息详情")
    public String getReportDetails(@PathVariable("boatName") String boatName) {
        List<BoatReportVo> voList = service.getReportDetails(boatName);
        if (CollectionUtils.isEmpty(voList)) {
            return new ResultMsg<String>().setCode(ResultState.ERROR_NEEDDATA).setMsg("船舶保港信息暂无数据").toJSON();
        }
        List<BoatReportVo> result = new ArrayList<>(5);
        for (int i = 0; i < 5; i++) {
            result.add(voList.get(i));
        }
        return new ResultMsg<String>()
                .setCode(ResultState.SUCCESS)
                .setData(JSONObject.toJSONString(result))
                .toJSON();
    }

    @GetMapping("getSBYJReportDetails/{reportId}")
    @ApiOperation("申报预警信息详情")
    public String getSBYJReportDetails(@PathVariable("reportId") String reportId) {
        Optional<SBYJReportDetailVo> voOptional = service.getSBYJReportDetails(reportId);
        if (!voOptional.isPresent()) {
            return new ResultMsg<String>().setCode(ResultState.ERROR_NEEDDATA).setMsg("申报预警信息详情信息暂无数据").toJSON();
        }
        return new ResultMsg<String>()
                .setCode(ResultState.SUCCESS)
                .setData(JSONObject.toJSONString(voOptional.get()))
                .toJSON();
    }

    @GetMapping("getSBYJEepReportDetails/{reportId}")
    @ApiOperation("申报预警船舶人员信息列表")
    public String getSBYJEepReportDetails(@PathVariable("reportId") String reportId) {
        List<SBYJEepReportDetailVo> voList = service.getSBYJEepReportDetails(reportId);
        if (CollectionUtils.isEmpty(voList)) {
            return new ResultMsg<String>().setCode(ResultState.ERROR_NEEDDATA).setMsg("申报预警船舶人员信息列表暂无数据").toJSON();
        }
        return new ResultMsg<String>()
                .setCode(ResultState.SUCCESS)
                .setData(JSONObject.toJSONString(voList))
                .toJSON();
    }
}
