package com.shanghaichengdi.ghjgitem.controller;

import static com.shanghaigeography.Util.DateUtils.dateToString;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSONObject;
import com.shanghaichengdi.ghjgitem.controller.model.req.BayonetDeviceReq;
import com.shanghaichengdi.ghjgitem.controller.model.res.BayonetDeviceRes;
import com.shanghaichengdi.ghjgitem.controller.model.res.BayonetDeviceTo;
import com.shanghaichengdi.ghjgitem.domain.BayonetDevice;
import com.shanghaichengdi.ghjgitem.domain.BayonetDeviceUseState;
import com.shanghaichengdi.ghjgitem.domain.DictList;
import com.shanghaichengdi.ghjgitem.enums.OutsideInterface;
import com.shanghaichengdi.ghjgitem.service.IBayonetDeviceService;
import com.shanghaichengdi.ghjgitem.util.CommonTools;
import com.shanghaichengdi.ghjgitem.vo.BayonetDeviceVo;
import com.shanghaigeography.entity.ResultMsg;
import com.shanghaigeography.eum.ResultState;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.io.File;
import java.text.ParseException;
import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

/**
 * @author gengaoliu
 */
@Api("卡口设备管理")
@RestController
@CrossOrigin
@RequestMapping("/ghjg/bayonetDevice")
@Slf4j
public class BayonetDeviceController {

  @Resource
  IBayonetDeviceService bayonetDeviceService;

  String ghjgFileService = OutsideInterface.GHJG_FILE_SERVICE.getUrlString();
  
  String ghjgFileLoadUrl = OutsideInterface.GHJG_FILE_LOAD.getUrlString();

  @PostMapping("/addOrUpdate")
  @ApiOperation("添加/修改卡口设备")
  public String addOrUpdate(@RequestBody BayonetDeviceReq req) {
    try {
      bayonetDeviceService.saveOrUpdate(BayonetDeviceReq.to(req));
    } catch (Exception e) {
      return new ResultMsg<String>().setCode(ResultState.ERROR).setMsg(e.getMessage()).toJSON();
    }
    return new ResultMsg<String>().setCode(ResultState.SUCCESS).setMsg("操作成功").toJSON();
  }

  @DeleteMapping("/delete/{id}")
  @ApiOperation("删除卡口设备")
  public String delete(@PathVariable("id") String id) {
    try {
      bayonetDeviceService.deleteByLogic(id);
    } catch (Exception e) {
      return new ResultMsg<String>().setCode(ResultState.ERROR).setMsg(e.getMessage()).toJSON();
    }
    return new ResultMsg<String>().setCode(ResultState.SUCCESS).setMsg("删除成功").toJSON();
  }

  @PostMapping("/list")
  @ApiOperation("卡口设备列表")
  public String getList(@RequestBody Map<String, Object> map) {
    List<BayonetDeviceVo> voList = bayonetDeviceService.getAllPage(map);
    voList.forEach(
        x -> {
          x.setUseStatus(BayonetDeviceUseState.getByValue(x.getUseStatus()).getDescription());
        });
    int count = CommonTools.getTotal(voList);
    if (CollectionUtils.isEmpty(voList)) {
      return new ResultMsg<String>()
          .setCode(ResultState.ERROR_NEEDDATA)
          .setMsg("暂无数据")
          .setData(null)
          .toJSON();
    }
    return new ResultMsg<String>()
        .setCode(ResultState.SUCCESS)
        .setMsg(String.valueOf(count))
        .setData(JSONObject.toJSONString(voList))
        .toJSON();
  }

  @ApiOperation("卡口设备列表导出")
  @PostMapping("/getExcel")
  public String getExcel(@RequestBody Map<String, Object> map) {
    String newye = dateToString(new Date(), "yyyy");
    long s = System.nanoTime();
    String tkk =
        ghjgFileLoadUrl
            + "supervise/shipIdentification/rectify/"
            + newye
            + "/"
            + dateToString(new Date(), "MM")
            + "/excel/";
    new File(tkk).mkdirs();
    String towofile = tkk + s + "卡口设备列表信息.xlsx";
    List<BayonetDeviceTo> toList = new ArrayList<>();
    map.replace("size", 50000);
    JSONObject j = JSONObject.parseObject(this.getList(map));
    if (!"无数据".equals(j.get("msg"))) {
      int i = 1;
      List<BayonetDeviceVo> bayonetDeviceVos =
          j.getJSONArray("data").toJavaList(BayonetDeviceVo.class);
      for (BayonetDeviceVo vo : bayonetDeviceVos) {
        BayonetDeviceTo bayonetTo = new BayonetDeviceTo();
        BeanUtils.copyProperties(vo, bayonetTo);
        bayonetTo.setNum(i);
        toList.add(bayonetTo);
        i++;
      }
    }
    if (toList.size() >= 8000) {
      return new ResultMsg<String>()
          .setCode(ResultState.ERROR_NEEDDATA)
          .setMsg("导出数量庞大，请选择条件以缩小导出范围")
          .toJSON();
    }
    EasyExcel.write(towofile, BayonetDeviceTo.class).sheet("卡口设备列表信息").doWrite(() -> toList);
    String pdfUrl = towofile.replace(ghjgFileLoadUrl.replace("/test", ""), (ghjgFileService + "supervise/"));
    return new ResultMsg<String>().setCode(ResultState.SUCCESS).setMsg(pdfUrl).toJSON();
  }

  @ApiOperation("设备类型列表")
  @GetMapping("/getType")
  public String getType() {
    List<DictList> dictLists = bayonetDeviceService.getUseState();
    if (CollectionUtils.isEmpty(dictLists)) {
      return new ResultMsg<String>()
          .setCode(ResultState.ERROR_NEEDDATA)
          .setMsg("暂无数据")
          .setData(null)
          .toJSON();
    }
    return new ResultMsg<String>()
        .setCode(ResultState.SUCCESS)
        .setData(JSONObject.toJSONString(dictLists))
        .toJSON();
  }

  @ApiOperation("使用状态列表")
  @GetMapping("/getUseState")
  public String getUseState() {
    List<DictList> dictLists = new ArrayList<>();
    List<BayonetDeviceUseState> collect =
        Arrays.stream(BayonetDeviceUseState.values()).collect(Collectors.toList());
    for (BayonetDeviceUseState bayonetDeviceUseState : collect) {
      DictList dictList = new DictList();
      dictList.setId(bayonetDeviceUseState.getValue());
      dictList.setName(bayonetDeviceUseState.getDescription());
      dictLists.add(dictList);
    }
    return new ResultMsg<String>()
        .setCode(ResultState.SUCCESS)
        .setData(JSONObject.toJSONString(dictLists))
        .toJSON();
  }

  @ApiOperation("使用状态列表")
  @GetMapping("/getById/{id}")
  public String getById(@PathVariable("id") String id) throws ParseException {
    BayonetDevice byId = bayonetDeviceService.getById(id);
    return new ResultMsg<String>()
        .setCode(ResultState.SUCCESS)
        .setData(JSONObject.toJSONString(BayonetDeviceRes.from(byId)))
        .toJSON();
  }
}
