package com.shanghaichengdi.ghjgitem.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.shanghaichengdi.ghjgitem.domain.RingData;
import com.shanghaichengdi.ghjgitem.service.IFlowMonitoringStatisticsService;
import com.shanghaichengdi.ghjgitem.util.CommonTools;
import com.shanghaichengdi.ghjgitem.vo.FlowMonitoringStatisticsVo;
import com.shanghaigeography.entity.ResultMsg;
import com.shanghaigeography.eum.ResultState;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

/**
 * @author gengaoliu
 */
@Api("流量监测统计")
@RestController
@CrossOrigin
@RequestMapping("/ghjg/flowMonitoringStatistics")
@Slf4j
public class FlowMonitoringStatisticsController extends BaseController {

    private final IFlowMonitoringStatisticsService service;

    public FlowMonitoringStatisticsController(IFlowMonitoringStatisticsService service) {
        this.service = service;
    }

    @PostMapping("getAllPage")
    @ApiOperation("获取流量监测统计列表")
    public String getAllPage(@RequestBody Map<String, Object> map) {
        List<FlowMonitoringStatisticsVo> voList = service.getAllPage(map);
        int count = CommonTools.getTotal(voList);
        return new ResultMsg<String>()
                .setCode(ResultState.SUCCESS)
                .setMsg(String.valueOf(count))
                .setData(JSON.toJSONString(voList))
                .toJSON();
    }

    @ApiOperation("流量监测统计报表导出")
    @PostMapping("GetExcel")
    @ResponseBody
    @ApiImplicitParams(
            @ApiImplicitParam(name = "流量监测统计报表导出", dataTypeClass = FlowMonitoringStatisticsVo.class))
    public String getExcel(@RequestBody Map<String, Object> map) {
        map.replace("size", 8000);
        JSONObject jsonData = JSON.parseObject(getAllPage(map));
        List<FlowMonitoringStatisticsVo> dataList =
                jsonData.getJSONArray("data").toJavaList(FlowMonitoringStatisticsVo.class);
        if (CollectionUtils.isEmpty(dataList)) {
            return new ResultMsg<String>().setCode(ResultState.ERROR_NEEDDATA).setMsg("暂无数据").toJSON();
        }
        String data = service.getExcel(dataList);
        return new ResultMsg<String>()
                .setCode(ResultState.SUCCESS)
                .setMsg("导出成功")
                .setData(data)
                .toJSON();
    }

    @PostMapping("getChart1")
    @ApiOperation("获取流量监测统计艘量饼图数据")
    public String getChart1(@RequestBody Map<String, Object> map) {
        List<RingData> chart1 = service.getChart1(map);
        if (CollectionUtils.isEmpty(chart1)) {
            return new ResultMsg<String>().setCode(ResultState.ERROR_NEEDDATA).setMsg("暂无数据").toJSON();
        }
        return new ResultMsg<String>()
                .setCode(ResultState.SUCCESS)
                .setData(JSON.toJSONString(chart1))
                .toJSON();
    }

    @PostMapping("getChart2")
    @ApiOperation("获取流量监测统计吨量饼图数据")
    public String getChart2(@RequestBody Map<String, Object> map) {
        List<RingData> chart2 = service.getChart2(map);
        if (CollectionUtils.isEmpty(chart2)) {
            return new ResultMsg<String>().setCode(ResultState.ERROR_NEEDDATA).setMsg("暂无数据").toJSON();
        }
        return new ResultMsg<String>()
                .setCode(ResultState.SUCCESS)
                .setData(JSON.toJSONString(chart2))
                .toJSON();
    }

    @PostMapping("getChart3")
    @ApiOperation("获取流量监测统计艘量柱状图数据")
    public String getChart3(@RequestBody Map<String, Object> map) {
        List<RingData> chart3 = service.getChart3(map);
        if (CollectionUtils.isEmpty(chart3)) {
            return new ResultMsg<String>().setCode(ResultState.ERROR_NEEDDATA).setMsg("暂无数据").toJSON();
        }
        return qtAtLast(chart3);
    }

    @PostMapping("getChart4")
    @ApiOperation("获取流量监测统计吨量柱状图数据")
    public String getChart4(@RequestBody Map<String, Object> map) {
        List<RingData> chart4 = service.getChart4(map);
        if (CollectionUtils.isEmpty(chart4)) {
            return new ResultMsg<String>().setCode(ResultState.ERROR_NEEDDATA).setMsg("暂无数据").toJSON();
        }
        return qtAtLast(chart4);
    }

    private String qtAtLast(List<RingData> chart4) {
        List<RingData> result = new ArrayList<>(chart4.size());
        AtomicReference<RingData> qt = new AtomicReference<>();
        chart4.forEach(
                x -> {
                    result.add(x);
                    if ("其他".equals(x.getName())) {
                        qt.set(x);
                        result.remove(x);
                    }
                });
        result.add(qt.get());
        return getResult(result);
    }

    private String getResult(List<RingData> chart4) {
        if (CollectionUtils.isEmpty(chart4)) {
            return new ResultMsg<String>().setCode(ResultState.ERROR_NEEDDATA).setMsg("暂无数据").toJSON();
        }
        HashMap<String, Object> res = new HashMap<>(chart4.size());
        List<String> name = new ArrayList<>();
        List<String> num = new ArrayList<>();
        chart4.forEach(
                ringData -> {
                    name.add(ringData.getName());
                    num.add(ringData.getValue());
                });
        res.put("name", name);
        res.put("value", num);
        return new ResultMsg<String>()
                .setCode(ResultState.SUCCESS)
                .setData(JSON.toJSONString(res))
                .toJSON();
    }
}
