package com.shanghaichengdi.ghjgitem.controller;

import static com.shanghaigeography.Util.DateUtils.dateToString;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSONObject;
import com.shanghaichengdi.ghjgitem.controller.model.res.ForecastOfIncomingShipsTo;
import com.shanghaichengdi.ghjgitem.enums.OutsideInterface;
import com.shanghaichengdi.ghjgitem.service.IForecastOfIncomingShipsService;
import com.shanghaichengdi.ghjgitem.util.CommonTools;
import com.shanghaichengdi.ghjgitem.vo.BoatPredictInfoVo;
import com.shanghaichengdi.ghjgitem.vo.ForecastBoatInfoVo;
import com.shanghaichengdi.ghjgitem.vo.ForecastOfIncomingShipsVo;
import com.shanghaichengdi.ghjgitem.vo.ReportBoatInfo;
import com.shanghaigeography.entity.ResultMsg;
import com.shanghaigeography.eum.ResultState;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.io.File;
import java.util.*;
import javax.annotation.Resource;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

/**
 * @author gengaoliu
 */
@Api("进港船舶预测")
@RestController
@CrossOrigin
@RequestMapping("/ghjg/forecastOfIncomingShips")
public class ForecastOfIncomingShipsController {
  @Resource IForecastOfIncomingShipsService forecastOfIncomingShipsService;

  String ghjgFileservice = OutsideInterface.GHJG_FILE_SERVICE.getUrlString();

  String url = OutsideInterface.GHJG_FILE_LOAD.getUrlString();

  @PostMapping("/getAllPage")
  @ApiOperation("进港船舶预测报表")
  public String getAllPage(@RequestBody Map<String, Object> map) {
    List<ForecastOfIncomingShipsVo> voList = forecastOfIncomingShipsService.getAllPage(map);
    int count = CommonTools.getTotal(voList);
    if (CollectionUtils.isEmpty(voList)) {
      return new ResultMsg<String>()
          .setCode(ResultState.ERROR_NEEDDATA)
          .setMsg("暂无数据")
          .setData(null)
          .toJSON();
    }
    return new ResultMsg<String>()
        .setCode(ResultState.SUCCESS)
        .setMsg(String.valueOf(count))
        .setData(JSONObject.toJSONString(voList))
        .toJSON();
  }

  @ApiOperation("进港船舶预测导出excel")
  @PostMapping("/getExcel")
  public String getExcel(@RequestBody Map<String, Object> map) {
    String newye = dateToString(new Date(), "yyyy");
    long s = System.nanoTime();
    String tkk =
        url
            + "supervise/shipIdentification/rectify/"
            + newye
            + "/"
            + dateToString(new Date(), "MM")
            + "/excel/";
    new File(tkk).mkdirs();
    String towofile = tkk + s + "进港船舶预测报表.xlsx";
    List<ForecastOfIncomingShipsTo> toList = new ArrayList<>();
    map.replace("size", 50000);
    JSONObject j = JSONObject.parseObject(this.getAllPage(map));
    if (!"无数据".equals(j.get("msg"))) {
      int i = 1;
      List<ForecastOfIncomingShipsVo> forecastOfIncomingShipsVoList =
          j.getJSONArray("data").toJavaList(ForecastOfIncomingShipsVo.class);
      for (ForecastOfIncomingShipsVo vo : forecastOfIncomingShipsVoList) {
        ForecastOfIncomingShipsTo forecastOfIncomingShipsTo = new ForecastOfIncomingShipsTo();
        BeanUtils.copyProperties(vo, forecastOfIncomingShipsTo);
        forecastOfIncomingShipsTo.setNum(i);
        toList.add(forecastOfIncomingShipsTo);
        i++;
      }
    }
    if (toList.size() >= 8000) {
      return new ResultMsg<String>()
          .setCode(ResultState.ERROR_NEEDDATA)
          .setMsg("导出数量庞大，请选择条件以缩小导出范围")
          .toJSON();
    }
    EasyExcel.write(towofile, ForecastOfIncomingShipsTo.class)
        .sheet("非现场执法报表信息")
        .doWrite(() -> toList);
    String pdfUrl = towofile.replace(url.replace("/test", ""), (ghjgFileservice + "supervise/"));
    return new ResultMsg<String>().setCode(ResultState.SUCCESS).setMsg(pdfUrl).toJSON();
  }

  @ApiOperation("船舶基本信息")
  @GetMapping("/getBasicInfo/{mmsi}/{boatName}")
  public String getBasicInfo(
      @PathVariable("mmsi") String mmsi, @PathVariable("boatName") String boatName) {
    Optional<ForecastBoatInfoVo> voOptional =
        forecastOfIncomingShipsService.findBoatBasicInfo(mmsi, boatName);
    if (!voOptional.isPresent()) {
      return new ResultMsg<String>()
          .setCode(ResultState.ERROR_NEEDDATA)
          .setMsg("船舶基本信息暂无数据")
          .toJSON();
    }
    return new ResultMsg<String>()
        .setCode(ResultState.SUCCESS)
        .setData(JSONObject.toJSONString(voOptional.get()))
        .toJSON();
  }

  @ApiOperation("船舶预测信息")
  @GetMapping("/getPredictInfo/{mmsi}/{boatName}")
  public String getPredictInfo(
      @PathVariable("mmsi") String mmsi, @PathVariable("boatName") String boatName) {
    Optional<BoatPredictInfoVo> voOptional =
        forecastOfIncomingShipsService.getPredictInfo(mmsi, boatName);
    if (!voOptional.isPresent()) {
      return new ResultMsg<String>()
          .setCode(ResultState.ERROR_NEEDDATA)
          .setMsg("船舶预测信息暂无数据")
          .toJSON();
    }
    BoatPredictInfoVo vo = voOptional.get();
    vo.setSpeed(Optional.ofNullable(vo.getSpeed()).orElse("0"));
    return new ResultMsg<String>()
        .setCode(ResultState.SUCCESS)
        .setData(JSONObject.toJSONString(vo))
        .toJSON();
  }

  @ApiOperation("船舶到港信息")
  @GetMapping("/getPortInfo/{mmsi}/{boatName}")
  public String getPortInfo(
      @PathVariable("mmsi") String mmsi, @PathVariable("boatName") String boatName) {
    List<ReportBoatInfo> infoList = forecastOfIncomingShipsService.getPortInfo(mmsi, boatName);
    if (CollectionUtils.isEmpty(infoList)) {
      return new ResultMsg<String>()
          .setCode(ResultState.ERROR_NEEDDATA)
          .setMsg("船舶到港信息暂无数据")
          .toJSON();
    }
    List<ReportBoatInfo> reportBoatInfos = new ArrayList<>(5);
    for (int i = 0; i < 5; i++) {
      reportBoatInfos.add(infoList.get(i));
    }
    return new ResultMsg<String>()
        .setCode(ResultState.SUCCESS)
        .setMsg(String.valueOf(reportBoatInfos.size()))
        .setData(JSONObject.toJSONString(reportBoatInfos))
        .toJSON();
  }
}
