package com.shanghaichengdi.ghjgitem.controller;

import static com.shanghaigeography.Util.DateUtils.dateToString;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSONObject;
import com.shanghaichengdi.ghjgitem.controller.model.res.OffSiteEnforcementTo;
import com.shanghaichengdi.ghjgitem.enums.OutsideInterface;
import com.shanghaichengdi.ghjgitem.service.IOffSiteEnforcementService;
import com.shanghaichengdi.ghjgitem.util.CommonTools;
import com.shanghaichengdi.ghjgitem.vo.OffSiteEnforcementVo;
import com.shanghaigeography.entity.ResultMsg;
import com.shanghaigeography.eum.ResultState;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.io.File;
import java.util.*;
import javax.annotation.Resource;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

/**
 * @author gengaoliu
 */
@Api("非现场执法")
@RestController
@CrossOrigin
@RequestMapping("/ghjg/offeSiteEnforcement")
public class OffSiteEnforcementController {
  @Resource
  IOffSiteEnforcementService offSiteEnforcementService;
  String ghjgFileservice = OutsideInterface.GHJG_FILE_SERVICE.getUrlString();
  String url = OutsideInterface.GHJG_FILE_LOAD.getUrlString();

  @PostMapping("/getAllPage")
  @ApiOperation("非现场执法报表")
  public String getAllPage(@RequestBody Map<String, Object> map) {
    List<OffSiteEnforcementVo> voList = offSiteEnforcementService.getAllPage(map);
    int count = CommonTools.getTotal(voList);
    if (CollectionUtils.isEmpty(voList)) {
      return new ResultMsg<String>()
          .setCode(ResultState.ERROR_NEEDDATA)
          .setMsg("暂无数据")
          .setData(null)
          .toJSON();
    }
    return new ResultMsg<String>()
        .setCode(ResultState.SUCCESS)
        .setMsg(String.valueOf(count))
        .setData(JSONObject.toJSONString(voList))
        .toJSON();
  }

  @ApiOperation("非现场执法报表导出")
  @PostMapping("/getExcel")
  public String getExcel(@RequestBody Map<String, Object> map) {
    String newye = dateToString(new Date(), "yyyy");
    long s = System.nanoTime();
    String tkk =
        url
            + "supervise/shipIdentification/rectify/"
            + newye
            + "/"
            + dateToString(new Date(), "MM")
            + "/excel/";
    new File(tkk).mkdirs();
    String towofile = tkk + s + "非现场执法报表信息.xlsx";
    List<OffSiteEnforcementTo> toList = new ArrayList<>();
    map.replace("size", 50000);
    JSONObject j = JSONObject.parseObject(this.getAllPage(map));
    if (!"无数据".equals(j.get("msg"))) {
      int i = 1;
      List<OffSiteEnforcementVo> enforcementVoList =
          j.getJSONArray("data").toJavaList(OffSiteEnforcementVo.class);
      for (OffSiteEnforcementVo vo : enforcementVoList) {
        OffSiteEnforcementTo enforcementTo = new OffSiteEnforcementTo();
        BeanUtils.copyProperties(vo, enforcementTo);
        enforcementTo.setNum(i);
        toList.add(enforcementTo);
        i++;
      }
    }
    if (toList.size() >= 8000) {
      return new ResultMsg<String>()
          .setCode(ResultState.ERROR_NEEDDATA)
          .setMsg("导出数量庞大，请选择条件以缩小导出范围")
          .toJSON();
    }
    EasyExcel.write(towofile, OffSiteEnforcementTo.class).sheet("非现场执法报表信息").doWrite(() -> toList);
    String pdfUrl = towofile.replace(url.replace("/test", ""), (ghjgFileservice + "supervise/"));
    return new ResultMsg<String>().setCode(ResultState.SUCCESS).setMsg(pdfUrl).toJSON();
  }


  @GetMapping("getPicture/{warnCode}")
  @ApiOperation("获取【船舶监控")
  @ResponseBody
  public String GetPicture(@PathVariable("warnCode") String warnCode) {
    return offSiteEnforcementService.getPicture(warnCode);
  }
}
