package com.shanghaichengdi.ghjgitem.controller;

import static com.shanghaigeography.Util.DateUtils.dateToString;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSONObject;
import com.shanghaichengdi.ghjgitem.controller.model.res.BoatInfoVoRes;
import com.shanghaichengdi.ghjgitem.controller.model.res.ShipIdentificationTo;
import com.shanghaichengdi.ghjgitem.domain.ShipBaseInfo;
import com.shanghaichengdi.ghjgitem.domain.ShipReportInfo;
import com.shanghaichengdi.ghjgitem.enums.OutsideInterface;
import com.shanghaichengdi.ghjgitem.service.ShipIdentificationService;
import com.shanghaichengdi.ghjgitem.vo.BoatInfoVo;
import com.shanghaichengdi.ghjgitem.vo.ShipIdentificationVo;
import com.shanghaichengdi.ghjgitem.vo.ShipPerceptionInfoVo;
import com.shanghaigeography.entity.ResultMsg;
import com.shanghaigeography.eum.ResultState;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import java.io.File;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

/**
 * @author gengaoliu
 */
@Api("船舶通航感知")
@Controller
@CrossOrigin
@RequestMapping("/ghjg/shipIdentification")
public class ShipIdentificationController {
  @Autowired ShipIdentificationService shipIdentificationService;
  String ghjgFileservice = OutsideInterface.GHJG_FILE_SERVICE.getUrlString();
  String url = OutsideInterface.GHJG_FILE_LOAD.getUrlString();

  @PostMapping("GetAllPage")
  @ApiOperation("获取全部【列表数据】")
  @ResponseBody
  public String GetAllPage(@RequestBody Map<String, Object> map) {
    return shipIdentificationService.GetAllPage(map);
  }

  @PostMapping("GetBayonetName")
  @ApiOperation("获取【卡口名称】字典")
  @ResponseBody
  public String GetBayonetName(@RequestBody Map<String, Object> map) {
    return shipIdentificationService.GetAllBayonet((String) map.get("name"));
  }

  @GetMapping("GetPicture/{evenCode}")
  @ApiOperation("获取【船舶监控】")
  @ResponseBody
  public String GetPicture(@PathVariable("evenCode") String evenCode) {
    return shipIdentificationService.GetPicture(evenCode);
  }

  /**
   * 文件导出
   *
   * @param map
   * @return
   */
  @ApiOperation("船舶通航感知信息导出")
  @PostMapping("GetExcel")
  @ResponseBody
  @ApiImplicitParams(
      @ApiImplicitParam(name = "船舶通航感知信息导出", dataTypeClass = ShipIdentificationVo.class))
  public String GetExcel(@RequestBody Map<String, Object> map) {
    String newye = dateToString(new Date(), "yyyy");
    long s = System.nanoTime();
    String tkk =
        url
            + "supervise/shipIdentification/rectify/"
            + newye
            + "/"
            + dateToString(new Date(), "MM")
            + "/excel/";
    new File(tkk).mkdirs();
    String towofile = tkk + s + "船舶通航感知信息.xlsx";
    List<ShipIdentificationTo> objAppointmentListTos = new LinkedList<>();
    map.replace("size", 50000);
    JSONObject j = JSONObject.parseObject(this.GetAllPage(map));
    if ("无数据".equals(j.get("msg"))) {
    } else {
      List<ShipIdentificationVo> objVehicleDeclares =
          j.getJSONArray("data").toJavaList(ShipIdentificationVo.class);
      for (int i = 0; i < objVehicleDeclares.size(); i++) {
        ShipIdentificationVo a = objVehicleDeclares.get(i);
        ShipIdentificationTo to = new ShipIdentificationTo();
        to.setNum(i + 1);
        to.setBoatName(a.getBoatName());
        to.setAnalysisName(a.getAnalysisName());
        to.setAnalysisTime(a.getAnalysisTime());
        to.setBayonetName(a.getBayonetName());
        to.setMmsi(a.getMmsi());
        to.setShipType(a.getShipType());
        to.setLength(a.getLength());
        to.setWidth(a.getWidth());
        to.setHeight(a.getHeight());
        to.setDirection(a.getDirection());
        to.setZd(a.getZd());
        to.setVelocity(a.getVelocity());
        to.setBoatId(a.getBoatId());
        to.setLoad(a.getLoad());
        objAppointmentListTos.add(to);
      }
    }
    if (objAppointmentListTos.size() >= 8000) {
      return new ResultMsg<String>()
          .setCode(ResultState.ERROR_NEEDDATA)
          .setMsg("导出数量庞大，请选择条件以缩小导出范围")
          .toJSON();
    }
    EasyExcel.write(towofile, ShipIdentificationTo.class)
        .sheet("船舶通航感知信息")
        .doWrite(() -> objAppointmentListTos);

    String pdfurl = towofile.replace(url.replace("/test", ""), (ghjgFileservice + "supervise/"));
    return new ResultMsg<String>().setCode(ResultState.SUCCESS).setMsg(pdfurl).toJSON();
  }

  @GetMapping("getBoatInfo/{shipId}")
  @ApiOperation("船舶通航感知查看船舶信息")
  @ResponseBody
  public String getSBYJEepReportDetails(@PathVariable("shipId") String shipId) {
    Optional<BoatInfoVo> voOptional = shipIdentificationService.getBoatInfo(shipId);
    if (!voOptional.isPresent()) {
      return new ResultMsg<String>()
          .setCode(ResultState.ERROR_NEEDDATA)
          .setMsg("船舶通航感知查看船舶信息暂无数据")
          .toJSON();
    }
    BoatInfoVo vo = voOptional.get();
    BoatInfoVoRes res = BoatInfoVoRes.from(vo);
    return new ResultMsg<String>()
        .setCode(ResultState.SUCCESS)
        .setData(JSONObject.toJSONString(res))
        .toJSON();
  }

  @GetMapping("getBoatInfoMore/{shipId}")
  @ApiOperation("船舶通航感知查看船舶信息(查看更多)")
  @ResponseBody
  public String getBoatInfoMore(@PathVariable("shipId") String shipId) {
    Optional<ShipBaseInfo> voOptional = shipIdentificationService.getBoatInfoMore(shipId);
    if (!voOptional.isPresent()) {
      return new ResultMsg<String>()
          .setCode(ResultState.ERROR_NEEDDATA)
          .setMsg("船舶通航感知查看船舶信息(更多详情)暂无数据")
          .toJSON();
    }
    ShipBaseInfo baseInfo = voOptional.get();
    if (Objects.equals(baseInfo.getShipRegionType(), "1")) {
      baseInfo.setShipRegionType("海船");
    } else if (Objects.equals(baseInfo.getShipRegionType(), "0")) {
      baseInfo.setShipRegionType("内河船");
    }
    return new ResultMsg<String>()
        .setCode(ResultState.SUCCESS)
        .setData(JSONObject.toJSONString(baseInfo))
        .toJSON();
  }

  @PutMapping("updateBoatInfo")
  @ApiOperation("修改船舶信息(查看更多)")
  @ResponseBody
  @Deprecated
  public String updateBoatInfo(@RequestBody ShipBaseInfo info) {
    try {
      shipIdentificationService.updateBoatInfo(info);
    } catch (Exception e) {
      e.printStackTrace();
      return new ResultMsg<String>()
          .setCode(ResultState.ERROR_NEEDDATA)
          .setMsg(e.getMessage())
          .toJSON();
    }
    return new ResultMsg<String>().setCode(ResultState.SUCCESS).setMsg("修改成功").toJSON();
  }

  @GetMapping("getPerceptionInfo/{evenCode}")
  @ApiOperation("船舶通航感知查看感知信息")
  @ResponseBody
  public String getPerceptionInfo(@PathVariable("evenCode") String evenCode) {
    Optional<ShipPerceptionInfoVo> voOptional =
        shipIdentificationService.getPerceptionInfo(evenCode);
    if (!voOptional.isPresent()) {
      return new ResultMsg<String>()
          .setCode(ResultState.ERROR_NEEDDATA)
          .setMsg("船舶通航感知感知信息暂无数据")
          .toJSON();
    }
    ShipPerceptionInfoVo vo = voOptional.get();
    vo.setSpeedString(vo.getSpeed()+"(公里)");
    vo.setLengthString(vo.getLength()+"(米)");
    vo.setWidthString(vo.getWidth()+"(米)");
    vo.setHeightString(vo.getHeight()+"(米)");
    return new ResultMsg<String>()
        .setCode(ResultState.SUCCESS)
        .setData(JSONObject.toJSONString(vo))
        .toJSON();
  }

  @GetMapping("getBoatReportInfo/{shipId}")
  @ApiOperation("船舶通航感知查看报告信息")
  @ResponseBody
  public String getBoatReportInfo(@PathVariable("shipId") String shipId) {
    List<ShipReportInfo> voList = shipIdentificationService.getBoatReportInfo(shipId);
    if (CollectionUtils.isEmpty(voList)) {
      return new ResultMsg<String>()
          .setCode(ResultState.ERROR_NEEDDATA)
          .setMsg("船舶通航感知查看报告信息暂无数据")
          .toJSON();
    }
    ShipReportInfo info = voList.get(0);
    return new ResultMsg<String>()
        .setCode(ResultState.SUCCESS)
        .setData(JSONObject.toJSONString(info))
        .toJSON();
  }
}
