package com.shanghaichengdi.ghjgitem.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.pagehelper.PageHelper;
import com.shanghaichengdi.ghjgitem.domain.Bayonet;
import com.shanghaichengdi.ghjgitem.domain.ShipBaseInfo;
import com.shanghaichengdi.ghjgitem.domain.ShipReportInfo;
import com.shanghaichengdi.ghjgitem.domain.Video;
import com.shanghaichengdi.ghjgitem.mapper.ShipIdentificationMapper;
import com.shanghaichengdi.ghjgitem.mapper.ShipReportInfoMapper;
import com.shanghaichengdi.ghjgitem.util.CommonTools;
import com.shanghaichengdi.ghjgitem.vo.BoatInfoVo;
import com.shanghaichengdi.ghjgitem.vo.ShipIdentificationVo;
import com.shanghaichengdi.ghjgitem.vo.ShipPerceptionInfoVo;
import com.shanghaigeography.entity.ResultMsg;
import com.shanghaigeography.eum.ResultState;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

/**
 * @author gengaoliu
 */
@Service
public class ShipIdentificationService {
  @Resource
  ShipIdentificationMapper mapper;

  @Resource ShipReportInfoMapper shipReportInfoMapper;

  // 获取所有数据
  public String GetAllPage(Map<String, Object> info) {
    PageHelper.startPage((int) info.get("current"), (int) info.get("size"));
    String capacity = String.valueOf(info.get("capacity"));
    info.replace("capacity", capacity);
    List<ShipIdentificationVo> objVehicleDeclares = mapper.getAllPage(info);
    int count = CommonTools.getTotal(objVehicleDeclares);
    return new ResultMsg<String>()
        .setCode(ResultState.SUCCESS)
        .setMsg(String.valueOf(count))
        .setData(JSON.toJSONString(objVehicleDeclares))
        .toJSON();
  }
  // 获取卡口字典
  public String GetAllBayonet(String name) {
    List<Bayonet> all = mapper.GetAllBayonet(name);
    if (all != null && all.size() > 0) {
      return new ResultMsg<String>()
          .setCode(ResultState.SUCCESS)
          .setData(JSONObject.toJSONString(all))
          .toJSON();
    }
    return new ResultMsg<String>().setCode(ResultState.SUCCESS).toJSON();
  }

  /** 获取监控图片 */
  public String GetPicture(String code) {
    Optional<Video> videoOptional = Optional.ofNullable(mapper.GetPicture(code));
    if (!videoOptional.isPresent()) {
      return new ResultMsg<String>()
          .setCode(ResultState.ERROR_NEEDDATA)
          .setMsg("该船舶没有对应的图片信息")
          .toJSON();
    }
    String photoName = videoOptional.get().getPhotoname();
    String[] split = photoName.split(";");
    List<String> photoUrl = Arrays.stream(split).collect(Collectors.toList());
    return new ResultMsg<String>()
        .setCode(ResultState.SUCCESS)
        .setData(JSONObject.toJSONString(photoUrl))
        .toJSON();
  }

  public Optional<BoatInfoVo> getBoatInfo(String shipId) {
    Objects.requireNonNull(shipId, "shipId cannot be null");
    return Optional.ofNullable(mapper.getBoatInfo(shipId));
  }

  public Optional<ShipBaseInfo> getBoatInfoMore(String shipId) {
    Objects.requireNonNull(shipId, "shipId cannot be null");
    LambdaQueryWrapper<ShipBaseInfo> queryWrapper =
        Wrappers.<ShipBaseInfo>lambdaQuery().eq(ShipBaseInfo::getShipId, shipId);
    return Optional.ofNullable(mapper.selectOne(queryWrapper));
  }

  public void updateBoatInfo(ShipBaseInfo info) {
    Objects.requireNonNull(info, "the Entity cannot be null");
    if (Objects.equals(info.getShipRegionType(), "海船")) {
      info.setShipRegionType("1");
    } else if (Objects.equals(info.getShipRegionType(), "内河船")) {
      info.setShipRegionType("0");
    }
    mapper.updateShipBaseInfo(info);
  }

  public Optional<ShipPerceptionInfoVo> getPerceptionInfo(String evenCode) {
    Objects.requireNonNull(evenCode, "evenCode cannot be null");
    return Optional.ofNullable(mapper.getPerceptionInfo(evenCode).get(0));
  }

  public List<ShipReportInfo> getBoatReportInfo(String shipId) {
    Objects.requireNonNull(shipId, "shipId cannot be null");
    LambdaQueryWrapper<ShipReportInfo> queryWrapper =
        Wrappers.<ShipReportInfo>lambdaQuery()
            .eq(ShipReportInfo::getShipId, shipId)
            .orderBy(StringUtils.hasLength(shipId), false, ShipReportInfo::getLastUpdateDate);
    return shipReportInfoMapper.selectList(queryWrapper);
  }
}
