package com.shanghaichengdi.ghjgitem.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageHelper;
import com.shanghaichengdi.ghjgitem.controller.model.res.FlowMonitoringStatisticsTo;
import com.shanghaichengdi.ghjgitem.domain.RingData;
import com.shanghaichengdi.ghjgitem.enums.FileEum;
import com.shanghaichengdi.ghjgitem.enums.OutsideInterface;
import com.shanghaichengdi.ghjgitem.mapper.FlowMonitoringStatisticsMapper;
import com.shanghaichengdi.ghjgitem.service.IFlowMonitoringStatisticsService;
import com.shanghaichengdi.ghjgitem.util.FileUtilString;
import com.shanghaichengdi.ghjgitem.vo.FlowMonitoringStatisticsVo;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

/**
 * @author gengaoliu
 */
@Service
@Slf4j
public class FlowMonitoringStatisticsServiceImpl implements IFlowMonitoringStatisticsService {
  @Resource FlowMonitoringStatisticsMapper mapper;
  private final FileUtilString fileUtilString;

  String ghjgFileLoad = OutsideInterface.GHJG_FILE_LOAD.getUrlString();

  public FlowMonitoringStatisticsServiceImpl(FileUtilString fileUtilString) {
    this.fileUtilString = fileUtilString;
  }

  @Override
  public List<FlowMonitoringStatisticsVo> getAllPage(Map<String, Object> info) {
    PageHelper.startPage((int) info.get("current"), (int) info.get("size"));

    return mapper.getAllPage(info);
  }

  @Override
  public String getExcel(List<FlowMonitoringStatisticsVo> dataList) {
    String path = "";
    path = this.getFilePath();
    List<FlowMonitoringStatisticsTo> targetList = new ArrayList<>(dataList.size());
    dataList.forEach(
        x -> {
          FlowMonitoringStatisticsTo statisticsTo = new FlowMonitoringStatisticsTo();
          statisticsTo.setBayonetName(x.getBayonetName());
          statisticsTo.setTotal(x.getTotal());
          statisticsTo.setOutCount(x.getOutCount());
          statisticsTo.setInCount(x.getInCount());
          statisticsTo.setDryShip(x.getDryShip());
          statisticsTo.setOilShip(x.getOilShip());
          statisticsTo.setBulkShip(x.getBulkShip());
          statisticsTo.setBulkChemicalShip(x.getBulkChemicalShip());
          statisticsTo.setBulkCementShip(x.getBulkCementShip());
          statisticsTo.setMultipurposeShip(x.getMultipurposeShip());
          statisticsTo.setOther(x.getOther());
          statisticsTo.setZd(x.getZd());
          targetList.add(statisticsTo);
        });
    log.info("targetList--------->{}", targetList);
    String fileName =
        String.format(
            "%s%s%s%s", path, System.currentTimeMillis(), "流量监测统计报表", FileEum.XLSX.getCode());
    log.info("文件上传名称为-------->{}", fileName);
    EasyExcel.write(fileName, FlowMonitoringStatisticsTo.class)
        .sheet("流量监测统计报表")
        .doWrite(() -> targetList);
    path = fileUtilString.getFileUrl(new File(fileName));
    return JSON.toJSONString(path);
  }

  @Override
  public List<RingData> getChart1(Map<String, Object> map) {
    return mapper.getChart1(map);
  }

  @Override
  public List<RingData> getChart2(Map<String, Object> map) {
    return mapper.getChart2(map);
  }

  @Override
  public List<RingData> getChart3(Map<String, Object> map) {
    return mapper.getChart3(map);
  }

  @Override
  public List<RingData> getChart4(Map<String, Object> map) {
    return mapper.getChart4(map);
  }

  String getFilePath() {
    File file = new File(ghjgFileLoad + "cargoDeclare/");
    if (!file.exists()) {
      file.mkdirs();
    }
    return ghjgFileLoad + "cargoDeclare/";
  }
}
