package com.shanghaichengdi.ghjgitem.service.impl;

import com.github.pagehelper.PageHelper;
import com.shanghaichengdi.ghjgitem.mapper.ForecastOfIncomingShipsMapper;
import com.shanghaichengdi.ghjgitem.service.IForecastOfIncomingShipsService;
import com.shanghaichengdi.ghjgitem.vo.BoatPredictInfoVo;
import com.shanghaichengdi.ghjgitem.vo.ForecastBoatInfoVo;
import com.shanghaichengdi.ghjgitem.vo.ForecastOfIncomingShipsVo;
import com.shanghaichengdi.ghjgitem.vo.ReportBoatInfo;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import javax.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

/**
 * @author gengaoliu
 */
@Service
public class ForecastOfIncomingShipsServiceImpl implements IForecastOfIncomingShipsService {

  private static String MODEL = "名称为:%s的船舶%s经过卡口区域，系统%s到该船舶本航次进出港报告信息";
  @Resource ForecastOfIncomingShipsMapper mapper;

  @Override
  public List<ForecastOfIncomingShipsVo> getAllPage(Map<String, Object> info) {
    PageHelper.startPage((int) info.get("current"), (int) info.get("size"));
    Optional<String> reportStatus = Optional.ofNullable(String.valueOf(info.get("reportStatus")));
    if (reportStatus.isPresent() && String.valueOf(1).equals(reportStatus.get())) {
      info.remove("reportStatus");
      info.put("reportStatus1", "1");
    } else if (reportStatus.isPresent() && String.valueOf(2).equals(reportStatus.get())) {
      info.remove("reportStatus");
      info.put("reportStatus2", "2");
    }
    return mapper.getList(info);
  }

  @Override
  public Optional<ForecastBoatInfoVo> findBoatBasicInfo(String mmsi, String boatName) {
    Objects.requireNonNull(mmsi, "mmsi cannot be null");
    Objects.requireNonNull(boatName, "boatName cannot be null");
    return Optional.ofNullable(mapper.findBoatBasicInfo(mmsi, boatName).get(0));
  }

  @Override
  public Optional<BoatPredictInfoVo> getPredictInfo(String mmsi, String boatName) {
    Objects.requireNonNull(mmsi, "mmsi cannot be null");
    Objects.requireNonNull(boatName, "boatName cannot be null");
    List<BoatPredictInfoVo> predictInfoList = mapper.findPredictInfoList(mmsi);
    if (CollectionUtils.isEmpty(predictInfoList)) {
      return Optional.empty();
    }
    BoatPredictInfoVo value = predictInfoList.get(0);
    String format =
        String.format(
            MODEL,
            Optional.of(boatName).orElse("xxx"),
            Optional.ofNullable(value.getDirection()).orElse("xx"),
            Optional.ofNullable(value.getStatus()).orElse("xxx"));
    value.setDescription(format);
    return Optional.ofNullable(value);
  }

  @Override
  public List<ReportBoatInfo> getPortInfo(String mmsi, String boatName) {
    Objects.requireNonNull(mmsi, "mmsi cannot be null");
    Objects.requireNonNull(boatName, "boatName cannot be null");
    return mapper.findReportBoatInfoList(mmsi, boatName);
  }
}
